@@ -3,12 +3,14 @@ module Stackctl.Colors
   ( Colors(..)
   , HasColorOption
   , getColorsStdout
+  , getColorsLogger
   , noColors
   ) where
 
 import Stackctl.Prelude
 
 import Blammo.Logging.Colors
+import Blammo.Logging.Logger
 import Stackctl.ColorOption (HasColorOption(..), colorHandle)
 
 -- | Return 'Colors' based on options and 'stdout'
@@ -24,5 +26,9 @@ getColorsHandle h = do
   c <- colorHandle h colorOption
   pure $ getColors c
 
+-- | Return 'Colors' consistent with the ambient 'Logger'
+getColorsLogger :: (MonadReader env m, HasLogger env) => m Colors
+getColorsLogger = view $ loggerL . to (getColors . getLoggerShouldColor)
+
 noColors :: Colors
 noColors = getColors False
