@@ -10,7 +10,7 @@ import Stackctl.Prelude
 
 import Stackctl.AWS
 import Stackctl.AWS.Scope
-import Stackctl.ColorOption
+import Stackctl.Colors
 import Stackctl.DirectoryOption
 import Stackctl.FilterOption
 import Stackctl.Spec.Capture
@@ -46,11 +46,11 @@ capture = Subcommand
   }
 
 changes
-  :: ( HasAwsScope env
+  :: ( HasLogger env
+     , HasAwsScope env
      , HasAwsEnv env
      , HasDirectoryOption env
      , HasFilterOption env
-     , HasColorOption env
      )
   => Subcommand ChangesOptions env
 changes = Subcommand
@@ -66,7 +66,6 @@ deploy
      , HasAwsEnv env
      , HasDirectoryOption env
      , HasFilterOption env
-     , HasColorOption env
      )
   => Subcommand DeployOptions env
 deploy = Subcommand
